#!/usr/bin/python
# -*- coding:utf-8 -*-

import logging
import time

import os
import sys
libdir = os.path.dirname(os.path.realpath(__file__))
if os.path.exists(libdir):
    sys.path.append(libdir)

import config

CH_EN=[0x57,0x68,0x02,0x00,0x00,0x00]


class SCR:
    def __init__(self, Baudrate = 115200, dev = "/dev/ttyS0", data_mode = 1, address=0x47):
        self.address = address
        self.data_mode = data_mode  #1 :uart   0: i2c
        self.Baudrate = Baudrate
        self.dev = dev
        self.com = config.config(Baudrate , dev , data_mode, address)
    
    def SendCommand(self, Data):
        if(self.data_mode == 1):
            #Data[6] = '\0'
            #self.com.UART_SendString(Data);
            self.com.UART_SendnByte(Data,6)
        if(self.data_mode == 0):    # 0: i2c
            self.com.I2C_SendWord(Data[2],(Data[3]) | (Data[4]<<8))
        time.sleep(0.1)
    
    def SET_Check_Digit(self, Data):
        return  ((((Data[0]^Data[1])^Data[2])^Data[3])^Data[4])
    def SetMode(self, Mode):
        ch=[0x57,0x68,0x01,0x00,0x00,0x00]
        ch[4] = Mode&0x01
        ch[5] = self.SET_Check_Digit(ch)
        self.SendCommand(ch)

    def ChannelEnable(self, Channel):
        if(Channel == 1):
            CH_EN[4] = 0x01|CH_EN[4]
            CH_EN[5] = self.SET_Check_Digit(CH_EN)
            self.SendCommand(CH_EN)
        elif(Channel == 2):
            CH_EN[4] = 0x02|CH_EN[4]
            CH_EN[5] = self.SET_Check_Digit(CH_EN)
            self.SendCommand(CH_EN)
    
    def ChannelDisable(self,Channel):
        if(Channel == 1):
            CH_EN[4] = 0xfe & CH_EN[4]
            CH_EN[5] = self.SET_Check_Digit(CH_EN)
            self.SendCommand(CH_EN)
        elif(Channel == 2):
            CH_EN[4] = 0xfd & CH_EN[4]
            CH_EN[5] = self.SET_Check_Digit(CH_EN)
            self.SendCommand(CH_EN)
        
    def VoltageRegulation(self, Channel,  Angle):
        Angle1=[0x57,0x68,0x03,0x00,0x00,0x00]
        Angle2=[0x57,0x68,0x04,0x00,0x00,0x00]
        if(Channel == 1):
            Angle1[4] = Angle
            Angle1[5] = self.SET_Check_Digit(Angle1)
            self.SendCommand(Angle1)
        elif(Channel == 2):
            Angle2[4] = Angle
            Angle2[5] = self.SET_Check_Digit(Angle2)
            self.SendCommand(Angle2)
        
    def GridFrequency(self, Hz):
        Frequency=[0x57,0x68,0x05,0x00,0x32,0x00]
        if(Hz == 50 or Hz ==60):
            Frequency[4] = Hz
            Frequency[5] = self.SET_Check_Digit(Frequency)
            self.SendCommand(Frequency)
        
    def Reset(self, Delay):
        ch=[0x57,0x68,0x06,0x00,0x00,0x00]
        ch[4] = Delay & 0xff
        ch[3] = Delay >> 8
        ch[5] = self.SET_Check_Digit(ch)
        self.SendCommand(ch)

        
    




#!/usr/bin/python
# -*- coding:utf-8 -*-
import RPi.GPIO as GPIO
import serial
import smbus
import time
import logging
# dev = "/dev/ttySC0"

logger = logging.getLogger(__name__)

class config(object):
    def __init__(ser, Baudrate = 115200, dev = "/dev/ttyS0", data_mode = 1, address=0x47):
        
        ser.data_mode = data_mode
        GPIO.setmode(GPIO.BCM)
        GPIO.setwarnings(False)
        if(data_mode == 1):
            logger.info("Triac Hat interface is set to UART")
            logger.info("Triac Hat device is at port %s" % (dev))
            logger.info("Triac Hat baudrate is %s" % (Baudrate))
            ser.dev = dev
            # write_timeout MUST be 0, otherwise the proccess will hang if trying to reach the wrong port
            ser.serial = serial.Serial(dev, Baudrate, write_timeout=0)
        elif(data_mode == 0):
            logger.info("Triac Hat interface is set to I2C")
            ser.i2c = smbus.SMBus(1)
            ser.address = address
            
    def UART_SendByte(ser, value): 
        ser.serial.write(value) 
    
    def UART_SendnByte(ser, value, Num):
        for i in range(0, Num):
            ser.UART_SendByte([value[i]]) 
    
    def UART_SendString(ser, value): 
        ser.serial.write(value.encode('ascii'))

    def UART_ReceiveByte(ser): 
        return ser.serial.read(1).decode("utf-8")

    def UART_ReceiveString(ser, value): 
        data = ser.serial.read(value)
        return data.decode("utf-8")
        
    def Uart_Set_Baudrate(ser, Baudrate):
        ser.serial = serial.Serial(ser.dev, Baudrate)
         
    def I2C_SendByte(ser, reg, value): 
        ser.i2c.write_byte_data(ser.address, reg, value) 
        
    def I2C_SendWord(ser, reg, value): 
        ser.i2c.write_word_data(ser.address, reg, value)     
        
         
         
         



# -*- coding: utf-8 -*-
import logging
import asyncio
from cbpi.api import *
import serial.tools.list_ports as lp
from TriacHat_2CH_Driver import SCR
import math

logger = logging.getLogger(__name__)

@parameters([Property.Select(label="Channel", options=[1,2], description="Select which channel you want to assign to this actor"),
             Property.Select(label="Interface", options=["UART", "I2C"], description="Select which interface your Triac Hat is using. (Deafult is UART)"),
             Property.Select(label="Device Port", options=[str(port.device) for port in lp.comports(True)], description="Choose the port in which your Triac Hat is connected."),
             Property.Select(label="Frequency", options=[50, 60], description="Line Frequency in Hz (Deafult is 50Hz)")])
class TriacHat(CBPiActor):

    @action("Set Power", parameters=[Property.Number(label="Power", configurable=True, description="Power Setting [0-100]")])
    async def setpower(self, Power=100, **kwargs):
        logging.info(Power)
        self.power = int(Power)
        if self.power < 0:
            self.power = 0
        if self.power > 100:
            self.power = 100
        await self.set_power(self.power)
    
    # Initializing plugin variables
    async def on_start(self):
        self.power = None
        self.ch = int(self.props.get("Channel"))
        self.interface = 0 if self.props.get("Interface") == "I2C" else 1
        self.dev = self.props.get("Device Port")
        self.freq = self.props.get("Frequncy", 50)
        self.switch = SCR.SCR(dev=self.dev, data_mode=self.interface)
        self.switch.GridFrequency(self.freq)
        self.switch.SetMode(1)
        self.state = False

    async def on(self, power = None):
        if power is not None:
            self.power = power
        else:
            self.power = 100
        await self.set_power(self.power)
        logger.info("Triac Hat actor %s ON - Channel %s - Power %s" % (self.id, self.ch, self.power))
        self.state = True

    async def off(self):
        await self.set_power(0)
        logger.info("Triac Hat actor %s OFF - Channel %s " % (self.id, self.ch))
        self.state = False
        
    async def set_power(self, power):
        self.power = power
        if self.state == True and power == 0:
            self.switch.ChannelDisable(self.ch)
        elif power > 0:
            if self.state == False:
                self.switch.ChannelEnable(self.ch)
             # To get the angle from a given power, I use the rounded value in degrees of arccos(1-(Power in Percents/50)).
            self.switch.VoltageRegulation(self.ch, round(math.degrees(math.acos(1-(power/50))))-1)
        await self.cbpi.actor.actor_update(self.id, power)
    
    def get_state(self):
        return self.state
    
    async def run(self):
        while (self.running) == True:
            await asyncio.sleep(1)


def setup(cbpi):
    cbpi.plugin.register("TriacHatActor", TriacHat)
    pass


from setuptools import setup

# read the contents of your README file
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(name='cbpi4-TriacHatActor',
      version='0.0.6',
      description='CraftBeerPi Plugin for controlling actor using Waveshare\'s 2CH Triac HAT for Raspberry Pi',
      author='Netanel Klein',
      author_email='netanel@netanelk.com',
      url='https://github.com/netanelklein/cbpi4-TriacHatActor',
      licence='GPLv3',
      include_package_data=True,
      package_data={
        # If any package contains *.txt or *.rst files, include them:
      '': ['*.txt', '*.rst', '*.yaml'],
      'cbpi4-TriacHatActor': ['*','*.txt', '*.rst', '*.yaml']},
      packages=['cbpi4-TriacHatActor', 'TriacHat_2CH_Driver'],
      install_requires=['cbpi>=4.0.0.56', 'RPi.GPIO', 'pyserial', 'smbus'],
      long_description=long_description,
      long_description_content_type='text/markdown'
     )
